// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

#include "commands/DriveWithJoysticks.h"
#include "Utils.h"

DriveWithJoysticks::DriveWithJoysticks(Chassis &chassis, frc::Joystick &joystick)
    : m_chassis{chassis}, m_joystick{joystick} {
  SetName("DriveWithJoysticks");
  AddRequirements(&m_chassis);
}

// Called when the command is initially scheduled.
void DriveWithJoysticks::Initialize() {}

// Called repeatedly when this Command is scheduled to run
void DriveWithJoysticks::Execute() {
    auto x = m_joystick.GetX();
    auto y = m_joystick.GetY();
    auto z = m_joystick.GetZ();

    x = utils::joy_expo(x, constants::joysticks::JOY_DRIVE_EXPO);
    y = utils::joy_expo(y, constants::joysticks::JOY_DRIVE_EXPO);
    z = utils::joy_expo(z, constants::joysticks::JOY_TURN_EXPO);

    auto forward = -y * 2_fps;
    auto left = -x * 2_fps;
    auto theta = z * 1.5_rad_per_s;

    m_chassis.drive_field_relative(forward, left, theta);
}

// Called once the command ends or is interrupted.
void DriveWithJoysticks::End(bool interrupted) {}

// Returns true when the command should end.
bool DriveWithJoysticks::IsFinished() {
  return false;
}
