// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

#pragma once

#include "Constants.h"

#include "AHRS.h"
#include <ctre/Phoenix.h>
#include <frc/geometry/Rotation2d.h>
#include <frc/geometry/Translation2d.h>
#include <frc/kinematics/SwerveDriveKinematics.h>
#include <frc/kinematics/SwerveModuleState.h>
#include <frc2/command/SubsystemBase.h>


class Chassis : public frc2::SubsystemBase {
  struct Pod {
    TalonSRX turn;
    TalonFX drive;
    double tmp;
  };

 public:
  Chassis();

  void zero_pod_encoders();
  void zero_gyro();

  void drive(frc::ChassisSpeeds speeds);
  void drive_field_relative(
    units::meters_per_second_t forward,
    units::meters_per_second_t left,
    units::radians_per_second_t theta);

 private:
  void configure_pod(Pod &pod);
  void set_pod(Pod &pod, frc::SwerveModuleState command);
  frc::Rotation2d get_pod_angle(Pod &pod);
  

 private:
  // Components (e.g. motor controllers and sensors) should generally be
  // declared private and exposed only through public methods.

  Pod m_lf_pod{{constants::can_ids::LF_TURN}, {constants::can_ids::LF_DRIVE}};
  Pod m_rf_pod{{constants::can_ids::RF_TURN}, {constants::can_ids::RF_DRIVE}};
  Pod m_lb_pod{{constants::can_ids::LB_TURN}, {constants::can_ids::LB_DRIVE}};
  Pod m_rb_pod{{constants::can_ids::RB_TURN}, {constants::can_ids::RB_DRIVE}};

  // AHRS m_gyro{frc::SPI::Port::kMXP};

  const frc::Translation2d LF_POD_LOCATION{ 
    constants::chassis::SWERVE_X_FROM_CENTER,
    constants::chassis::SWERVE_Y_FROM_CENTER};
  const frc::Translation2d RF_POD_LOCATION{
    constants::chassis::SWERVE_X_FROM_CENTER,
    -constants::chassis::SWERVE_Y_FROM_CENTER};
  const frc::Translation2d LB_POD_LOCATION{
    -constants::chassis::SWERVE_X_FROM_CENTER,
    constants::chassis::SWERVE_Y_FROM_CENTER};
  const frc::Translation2d RB_POD_LOCATION{
    -constants::chassis::SWERVE_X_FROM_CENTER,
    -constants::chassis::SWERVE_Y_FROM_CENTER};

  frc::SwerveDriveKinematics<4> m_kinematics{
    LF_POD_LOCATION,
    RF_POD_LOCATION,
    LB_POD_LOCATION,
    RB_POD_LOCATION
  };
};
